library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity core_math_tester is
end core_math_tester;

architecture Test of core_math_tester is
	signal clk : std_logic := '0';
	signal reset : std_logic := '0';
	signal result : unsigned(8 downto 0);
	signal done : std_logic := '0';
	signal a, c, d : unsigned(8 DOWNTO 0) := (others => '0');
	signal b, e : signed(8 DOWNTO 0) := (others => '0');
	constant INFINITE : unsigned := "011111111";
begin
	math: entity work.core_math port map (
		clk => clk,
		reset => reset,
		a => a,
		b => b,
		c => c,
		d => d,
		e => e,
		result => result,
		done => done
	);

	process
	begin
		loop
			wait for 10 ns;
			clk <= '1';
			wait for 10 ns;
			clk <= '0';
		end loop;
	end process;
	
	process
	begin
		a <= "010000000"; b <= "111111111"; c <= "000000000"; d <= "000011001"; e <= "000011110";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "010000000" report "128 + -1 * (0 - 24) / -30 = 128" severity error;

		a <= "001010001"; b <= "001001100"; c <= "000100010"; d <= "000010111"; e <= "000101101";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "0" & x"63" report "81 + 76 * (34 - 23) / 45 = 99 wrong" severity error;
		
		--1  e = 0
		a <= "001010001"; b <= "001001100"; c <= "000100010"; d <= "000010111"; e <= "000000000";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = INFINITE report "81 + 76 * (34 - 23) / 0 = 255 wrong" severity error;

		--2  c < d
		a <= "001010001"; b <= "001001100"; c <= "000010111"; d <= "000100010"; e <= "000101101";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		-- 63 is actually wrong. The intersection happens with negative t value, i.e. the ray
		-- goes backwards. Subtle but very important.
		assert result = INFINITE report "81 + 76 * (23 - 34) / 45 => 255 wrong" severity error;
		
        --3 
		a <= "000001010"; b <= "111111011"; c <= "000100010"; d <= "000010111"; e <= "000101101";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "0" & x"08" or result = "0" & x"09" report "10 + (-5) * (34 - 23) / 45 = 9 wrong" severity error;
		
        --4
		a <= "000001010"; b <= "111111011"; c <= "000010100"; d <= "000100000"; e <= "000000111";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = INFINITE report "10 + (-5) * (20 - 32) / 7 => INFINITE  wrong" severity error;
        
        --5
		a <= "000001010"; b <= "001000101"; c <= "000100000"; d <= "000010100"; e <= "111111000";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = INFINITE report "10 + 5 * (32 - 20) / (-8) = INFINITE wrong" severity error;

        --6 
		a <= "000001010"; b <= "011111111"; c <= "000100000"; d <= "000010100"; e <= "000000001";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = INFINITE report "10 + 255 * (32 - 20) / 1 = 255 wrong" severity error;

		a <= "011111111"; b <= "000110010"; c <= "011111110"; d <= "011111111"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 50 * ( 254 - 255 ) / 50 => 255" severity error;
		a <= "010000000"; b <= "101100100"; c <= "000110010"; d <= "000011001"; e <= "010010111";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "001100111" report "128 + -156 * ( 50 - 25 ) / 151 => 103" severity error;
		a <= "011111111"; b <= "000110010"; c <= "000110010"; d <= "011111111"; e <= "000001011";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 50 * ( 50 - 255 ) / 11 => 255" severity error;
		a <= "011111111"; b <= "101101001"; c <= "000000000"; d <= "011111111"; e <= "001110000";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + -151 * ( 0 - 255 ) / 112 => 255" severity error;
		a <= "010000000"; b <= "101101100"; c <= "001111101"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "128 + -148 * ( 125 - 0 ) / 50 => 255" severity error;
		a <= "011111111"; b <= "001001001"; c <= "010101111"; d <= "011111111"; e <= "101101110";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 73 * ( 175 - 255 ) / -146 => 255" severity error;
		a <= "000000000"; b <= "000110010"; c <= "001001011"; d <= "010000000"; e <= "101110000";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000010010" report "0 + 50 * ( 75 - 128 ) / -144 => 18" severity error;
		a <= "000011001"; b <= "000100011"; c <= "011111110"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011001010" report "25 + 35 * ( 254 - 0 ) / 50 => 202" severity error;
		a <= "011111111"; b <= "000110010"; c <= "000110010"; d <= "011111111"; e <= "010000111";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 50 * ( 50 - 255 ) / 135 => 255" severity error;
		a <= "000011001"; b <= "111111100"; c <= "001111101"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000001111" report "25 + -4 * ( 125 - 0 ) / 50 => 15" severity error;
		a <= "000000000"; b <= "000110010"; c <= "000000000"; d <= "000011001"; e <= "001100001";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "0 + 50 * ( 0 - 25 ) / 97 => 255" severity error;
		a <= "000110100"; b <= "101111101"; c <= "011111110"; d <= "000000000"; e <= "000101011";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "52 + -131 * ( 254 - 0 ) / 43 => 255" severity error;
		a <= "000000000"; b <= "000110010"; c <= "010101111"; d <= "010000000"; e <= "101111111";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "0 + 50 * ( 175 - 128 ) / -129 => 255" severity error;
		a <= "011111111"; b <= "110000001"; c <= "000110010"; d <= "011111111"; e <= "010011110";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + -127 * ( 50 - 255 ) / 158 => 255" severity error;
		a <= "011111111"; b <= "110000100"; c <= "011111110"; d <= "011111111"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + -124 * ( 254 - 255 ) / 50 => 255" severity error;
		a <= "000011001"; b <= "001111000"; c <= "001111101"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "25 + 120 * ( 125 - 0 ) / 50 => 255" severity error;
		a <= "011111111"; b <= "110001001"; c <= "001111101"; d <= "011111111"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + -119 * ( 125 - 255 ) / 50 => 255" severity error;
		a <= "000011001"; b <= "001010001"; c <= "001001011"; d <= "010000000"; e <= "110001011";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000111101" report "25 + 81 * ( 75 - 128 ) / -117 => 61" severity error;
		a <= "000000000"; b <= "000110010"; c <= "011111110"; d <= "000011001"; e <= "010110110";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000111110" report "0 + 50 * ( 254 - 25 ) / 182 => 62" severity error;
		a <= "011111111"; b <= "110010000"; c <= "000110010"; d <= "011111111"; e <= "000101010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + -112 * ( 50 - 255 ) / 42 => 255" severity error;
		a <= "000000000"; b <= "000110010"; c <= "000110010"; d <= "000011001"; e <= "010001111";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000001000" report "0 + 50 * ( 50 - 25 ) / 143 => 8" severity error;
		a <= "010000000"; b <= "110010101"; c <= "000000000"; d <= "000011001"; e <= "000000100";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "128 + -107 * ( 0 - 25 ) / 4 => 255" severity error;
		a <= "011111111"; b <= "110010111"; c <= "001111101"; d <= "011111111"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + -105 * ( 125 - 255 ) / 50 => 255" severity error;
		a <= "000011001"; b <= "111011101"; c <= "010101111"; d <= "010000000"; e <= "110011010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "25 + -35 * ( 175 - 128 ) / -102 => 255" severity error;
		a <= "011111111"; b <= "000110010"; c <= "001001011"; d <= "011111111"; e <= "110011100";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 50 * ( 75 - 255 ) / -100 => 255" severity error;
		a <= "011111111"; b <= "010100110"; c <= "011111110"; d <= "011111111"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 166 * ( 254 - 255 ) / 50 => 255" severity error;
		a <= "000000000"; b <= "000110010"; c <= "000110010"; d <= "000011001"; e <= "000011011";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000101110" report "0 + 50 * ( 50 - 25 ) / 27 => 46" severity error;
		a <= "011111111"; b <= "001111111"; c <= "001111101"; d <= "011111111"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 127 * ( 125 - 255 ) / 50 => 255" severity error;
		a <= "011111111"; b <= "000110010"; c <= "000000000"; d <= "011111111"; e <= "111110100";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 50 * ( 0 - 255 ) / -12 => 255" severity error;
		a <= "010000000"; b <= "110101000"; c <= "011111110"; d <= "000011001"; e <= "001011001";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "128 + -88 * ( 254 - 25 ) / 89 => 255" severity error;
		a <= "000111100"; b <= "000110010"; c <= "010101111"; d <= "000011000"; e <= "110101010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "60 + 50 * ( 175 - 24 ) / -86 => 255" severity error;
		a <= "010000000"; b <= "110101101"; c <= "000110010"; d <= "000011001"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "001010111" report "128 + -83 * ( 50 - 25 ) / 50 => 87" severity error;
		a <= "010000000"; b <= "110101111"; c <= "011111110"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "128 + -81 * ( 254 - 0 ) / 50 => 255" severity error;
		a <= "011111111"; b <= "000001011"; c <= "001111101"; d <= "011111111"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 11 * ( 125 - 255 ) / 50 => 255" severity error;
		a <= "010000000"; b <= "110110100"; c <= "001111101"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "128 + -76 * ( 125 - 0 ) / 50 => 255" severity error;
		a <= "000000000"; b <= "000011100"; c <= "001001011"; d <= "000111111"; e <= "110110111";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "0 + 28 * ( 75 - 63 ) / -73 => 255" severity error;
		a <= "011111111"; b <= "000110010"; c <= "011111110"; d <= "011111111"; e <= "001001001";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 50 * ( 254 - 255 ) / 73 => 255" severity error;
		a <= "010000000"; b <= "110111011"; c <= "000110010"; d <= "000011001"; e <= "010101110";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "001110111" report "128 + -69 * ( 50 - 25 ) / 174 => 119" severity error;
		a <= "011111111"; b <= "000110010"; c <= "000110010"; d <= "011111111"; e <= "000100010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 50 * ( 50 - 255 ) / 34 => 255" severity error;
		a <= "000010100"; b <= "111000000"; c <= "000000000"; d <= "011111110"; e <= "101111001";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "20 + -64 * ( 0 - 254 ) / -135 => 255" severity error;
		a <= "010000000"; b <= "111000011"; c <= "001111101"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "128 + -61 * ( 125 - 0 ) / 50 => 255" severity error;
		a <= "011111111"; b <= "001100000"; c <= "010101111"; d <= "011111111"; e <= "111000101";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 96 * ( 175 - 255 ) / -59 => 255" severity error;
		a <= "000000000"; b <= "000110010"; c <= "001001011"; d <= "010000000"; e <= "111001000";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000101111" report "0 + 50 * ( 75 - 128 ) / -56 => 47" severity error;
		a <= "000011001"; b <= "000111010"; c <= "011111110"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "25 + 58 * ( 254 - 0 ) / 50 => 255" severity error;
		a <= "001001000"; b <= "000110010"; c <= "000110010"; d <= "011111110"; e <= "101100010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "010001000" report "72 + 50 * ( 50 - 254 ) / -158 => 136" severity error;
		a <= "000011001"; b <= "000010011"; c <= "001111101"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "001001000" report "25 + 19 * ( 125 - 0 ) / 50 => 72" severity error;
		a <= "000000000"; b <= "000110010"; c <= "000000000"; d <= "000011001"; e <= "001111000";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "0 + 50 * ( 0 - 25 ) / 120 => 255" severity error;
		a <= "001001001"; b <= "111010100"; c <= "011111110"; d <= "000000000"; e <= "000010100";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "73 + -44 * ( 254 - 0 ) / 20 => 255" severity error;
		a <= "000000000"; b <= "000110010"; c <= "010101111"; d <= "010000000"; e <= "111010110";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "0 + 50 * ( 175 - 128 ) / -42 => 255" severity error;
		a <= "001001110"; b <= "111011000"; c <= "000110010"; d <= "011111110"; e <= "101001011";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000100001" report "78 + -40 * ( 50 - 254 ) / -181 => 33" severity error;
		a <= "011111111"; b <= "111011011"; c <= "011111110"; d <= "011111111"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + -37 * ( 254 - 255 ) / 50 => 255" severity error;
		a <= "000011001"; b <= "010001111"; c <= "001111101"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "25 + 143 * ( 125 - 0 ) / 50 => 255" severity error;
		a <= "011111111"; b <= "111100000"; c <= "001111101"; d <= "011111111"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + -32 * ( 125 - 255 ) / 50 => 255" severity error;
		a <= "000011001"; b <= "001101000"; c <= "001001011"; d <= "010000000"; e <= "111100010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011010000" report "25 + 104 * ( 75 - 128 ) / -30 => 208" severity error;
		a <= "000000000"; b <= "000110010"; c <= "011111110"; d <= "000011001"; e <= "111011101";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "0 + 50 * ( 254 - 25 ) / -35 => 255" severity error;
		a <= "000101000"; b <= "111100111"; c <= "000110010"; d <= "011111110"; e <= "110111111";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "40 + -25 * ( 50 - 254 ) / -65 => 255" severity error;
		a <= "000000000"; b <= "000110010"; c <= "000110010"; d <= "000011001"; e <= "010100110";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000000111" report "0 + 50 * ( 50 - 25 ) / 166 => 7" severity error;
		a <= "010000000"; b <= "111101100"; c <= "000000000"; d <= "000011001"; e <= "000011011";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "128 + -20 * ( 0 - 25 ) / 27 => 255" severity error;
		a <= "001100000"; b <= "111101110"; c <= "001111101"; d <= "001011010"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "001010100" report "96 + -18 * ( 125 - 90 ) / 50 => 84" severity error;
		a <= "000011001"; b <= "111110100"; c <= "010101111"; d <= "010000000"; e <= "111110001";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "25 + -12 * ( 175 - 128 ) / -15 => 255" severity error;
		a <= "010000010"; b <= "000110010"; c <= "001001011"; d <= "001011111"; e <= "111110011";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011001110" report "130 + 50 * ( 75 - 95 ) / -13 => 206" severity error;
		a <= "011111110"; b <= "101000011"; c <= "011111110"; d <= "000111100"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "254 + -189 * ( 254 - 60 ) / 50 => 255" severity error;
		a <= "000000000"; b <= "000110010"; c <= "000110010"; d <= "000011001"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000011001" report "0 + 50 * ( 50 - 25 ) / 50 => 25" severity error;
		a <= "011111110"; b <= "101101010"; c <= "001111101"; d <= "001001100"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "001101011" report "254 + -150 * ( 125 - 76 ) / 50 => 107" severity error;
		a <= "011111110"; b <= "111001110"; c <= "000000000"; d <= "001010000"; e <= "000001011";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "254 + -50 * ( 0 - 80 ) / 11 => 255" severity error;
		a <= "010000000"; b <= "111111111"; c <= "011111110"; d <= "000011001"; e <= "001110000";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "001111110" report "128 + -1 * ( 254 - 25 ) / 112 => 126" severity error;
		a <= "000101100"; b <= "000110010"; c <= "010101111"; d <= "010000001"; e <= "000000010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "44 + 50 * ( 175 - 129 ) / 2 => 255" severity error;
		a <= "010000000"; b <= "000000100"; c <= "000110010"; d <= "000011001"; e <= "001001001";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "010000001" report "128 + 4 * ( 50 - 25 ) / 73 => 129" severity error;
		a <= "010000000"; b <= "000000110"; c <= "011111110"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "010011110" report "128 + 6 * ( 254 - 0 ) / 50 => 158" severity error;
		a <= "011000101"; b <= "000100010"; c <= "001111101"; d <= "011111110"; e <= "111001110";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "197 + 34 * ( 125 - 254 ) / -50 => 255" severity error;
		a <= "010000000"; b <= "000001011"; c <= "001111101"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "010011011" report "128 + 11 * ( 125 - 0 ) / 50 => 155" severity error;
		a <= "000000000"; b <= "000000101"; c <= "001001011"; d <= "011000110"; e <= "000001110";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "0 + 5 * ( 75 - 198 ) / 14 => 255" severity error;
		a <= "001110111"; b <= "000110010"; c <= "011111110"; d <= "011111110"; e <= "110100000";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "001110111" report "119 + 50 * ( 254 - 254 ) / -96 => 119" severity error;
		a <= "010000000"; b <= "000010011"; c <= "000110010"; d <= "000011001"; e <= "111010101";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "128 + 19 * ( 50 - 25 ) / -43 => 255" severity error;
		a <= "011001000"; b <= "000110010"; c <= "000110010"; d <= "011111110"; e <= "111000111";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "200 + 50 * ( 50 - 254 ) / -57 => 255" severity error;
		a <= "010100001"; b <= "000010111"; c <= "000000000"; d <= "011111110"; e <= "101100010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011000101" report "161 + 23 * ( 0 - 254 ) / -158 => 197" severity error;
		a <= "010000000"; b <= "000011010"; c <= "001111101"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011000001" report "128 + 26 * ( 125 - 0 ) / 50 => 193" severity error;
		a <= "011111110"; b <= "110001001"; c <= "010101111"; d <= "010110101"; e <= "000011100";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "254 + -119 * ( 175 - 181 ) / 28 => 255" severity error;
		a <= "000000000"; b <= "000110010"; c <= "001001011"; d <= "010000000"; e <= "000011111";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "0 + 50 * ( 75 - 128 ) / 31 => 255" severity error;
		a <= "000011001"; b <= "001010001"; c <= "011111110"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "25 + 81 * ( 254 - 0 ) / 50 => 255" severity error;
		a <= "000111111"; b <= "000110010"; c <= "000110010"; d <= "011111110"; e <= "101001011";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "001110111" report "63 + 50 * ( 50 - 254 ) / -181 => 119" severity error;
		a <= "000011001"; b <= "000101010"; c <= "001111101"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "010000010" report "25 + 42 * ( 125 - 0 ) / 50 => 130" severity error;
		a <= "000000000"; b <= "000110010"; c <= "000000000"; d <= "000011001"; e <= "010001111";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "0 + 50 * ( 0 - 25 ) / 143 => 255" severity error;
		a <= "011111111"; b <= "000101011"; c <= "011111110"; d <= "011111111"; e <= "000000011";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 43 * ( 254 - 255 ) / 3 => 255" severity error;
		a <= "000000000"; b <= "000110010"; c <= "010101111"; d <= "010000000"; e <= "000101101";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000110100" report "0 + 50 * ( 175 - 128 ) / 45 => 52" severity error;
		a <= "010100001"; b <= "000110000"; c <= "000110010"; d <= "000000000"; e <= "000100100";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011100011" report "161 + 48 * ( 50 - 0 ) / 36 => 227" severity error;
		a <= "011111111"; b <= "000110010"; c <= "011111110"; d <= "011111111"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 50 * ( 254 - 255 ) / 50 => 255" severity error;
		a <= "000011001"; b <= "010100110"; c <= "001111101"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "25 + 166 * ( 125 - 0 ) / 50 => 255" severity error;
		a <= "011111111"; b <= "000110111"; c <= "001111101"; d <= "011111111"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 55 * ( 125 - 255 ) / 50 => 255" severity error;
		a <= "000011001"; b <= "001111111"; c <= "001001011"; d <= "010000000"; e <= "000111001";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "25 + 127 * ( 75 - 128 ) / 57 => 255" severity error;
		a <= "000000000"; b <= "000110010"; c <= "011111110"; d <= "000011001"; e <= "111110100";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "0 + 50 * ( 254 - 25 ) / -12 => 255" severity error;
		a <= "011111111"; b <= "000111110"; c <= "000110010"; d <= "011111111"; e <= "001011000";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 62 * ( 50 - 255 ) / 88 => 255" severity error;
		a <= "000000000"; b <= "000110010"; c <= "000110010"; d <= "000011001"; e <= "010111101";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000000110" report "0 + 50 * ( 50 - 25 ) / 189 => 6" severity error;
		a <= "010000000"; b <= "001000011"; c <= "000000000"; d <= "000011001"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "128 + 67 * ( 0 - 25 ) / 50 => 255" severity error;
		a <= "011101001"; b <= "001000101"; c <= "001111101"; d <= "001001100"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "233 + 69 * ( 125 - 76 ) / 50 => 255" severity error;
		a <= "000011001"; b <= "000001011"; c <= "010101111"; d <= "010000000"; e <= "001001000";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000100000" report "25 + 11 * ( 175 - 128 ) / 72 => 32" severity error;
		a <= "011111111"; b <= "000110010"; c <= "001001011"; d <= "011111111"; e <= "001001010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 50 * ( 75 - 255 ) / 74 => 255" severity error;
		a <= "000000000"; b <= "000011100"; c <= "011111110"; d <= "000101100"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "001110101" report "0 + 28 * ( 254 - 44 ) / 50 => 117" severity error;
		a <= "000000000"; b <= "000110010"; c <= "000110010"; d <= "000011001"; e <= "001001001";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000010001" report "0 + 50 * ( 50 - 25 ) / 73 => 17" severity error;
		a <= "011111110"; b <= "101010011"; c <= "001111101"; d <= "001000010"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000110010" report "254 + -173 * ( 125 - 66 ) / 50 => 50" severity error;
		a <= "011111111"; b <= "000110010"; c <= "000000000"; d <= "011111111"; e <= "000100010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 50 * ( 0 - 255 ) / 34 => 255" severity error;
		a <= "010000000"; b <= "001010110"; c <= "011111110"; d <= "000011001"; e <= "010000111";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "128 + 86 * ( 254 - 25 ) / 135 => 255" severity error;
		a <= "011111111"; b <= "000110010"; c <= "010101111"; d <= "011111111"; e <= "001011001";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 50 * ( 175 - 255 ) / 89 => 255" severity error;
		a <= "010000000"; b <= "001011011"; c <= "000110010"; d <= "000011001"; e <= "001100000";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "010010111" report "128 + 91 * ( 50 - 25 ) / 96 => 151" severity error;
		a <= "010000000"; b <= "001011110"; c <= "011111110"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "128 + 94 * ( 254 - 0 ) / 50 => 255" severity error;
		a <= "011111111"; b <= "000111001"; c <= "001111101"; d <= "011111111"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 57 * ( 125 - 255 ) / 50 => 255" severity error;
		a <= "010000000"; b <= "001100010"; c <= "001111101"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "128 + 98 * ( 125 - 0 ) / 50 => 255" severity error;
		a <= "011111111"; b <= "000010010"; c <= "001001011"; d <= "011111111"; e <= "001100101";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 18 * ( 75 - 255 ) / 101 => 255" severity error;
		a <= "011111111"; b <= "000110010"; c <= "011111110"; d <= "011111111"; e <= "001110111";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 50 * ( 254 - 255 ) / 119 => 255" severity error;
		a <= "010000000"; b <= "001101010"; c <= "000110010"; d <= "000011001"; e <= "111101100";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "128 + 106 * ( 50 - 25 ) / -20 => 255" severity error;
		a <= "011111111"; b <= "000110010"; c <= "000110010"; d <= "011111111"; e <= "001010000";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 50 * ( 50 - 255 ) / 80 => 255" severity error;
		a <= "011111111"; b <= "001101110"; c <= "000000000"; d <= "011111111"; e <= "010110101";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 110 * ( 0 - 255 ) / 181 => 255" severity error;
		a <= "010000000"; b <= "001110001"; c <= "001111101"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "128 + 113 * ( 125 - 0 ) / 50 => 255" severity error;
		a <= "011111111"; b <= "010001110"; c <= "010101111"; d <= "011111111"; e <= "001110011";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 142 * ( 175 - 255 ) / 115 => 255" severity error;
		a <= "000000000"; b <= "000110010"; c <= "001001011"; d <= "010000000"; e <= "001110110";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "0 + 50 * ( 75 - 128 ) / 118 => 255" severity error;
		a <= "000011001"; b <= "001101000"; c <= "011111110"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "25 + 104 * ( 254 - 0 ) / 50 => 255" severity error;
		a <= "000100010"; b <= "000110010"; c <= "000110010"; d <= "000000000"; e <= "000100100";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "001100111" report "34 + 50 * ( 50 - 0 ) / 36 => 103" severity error;
		a <= "000011001"; b <= "001000001"; c <= "001111101"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "010111011" report "25 + 65 * ( 125 - 0 ) / 50 => 187" severity error;
		a <= "000000000"; b <= "000110010"; c <= "000000000"; d <= "000011001"; e <= "010100110";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "0 + 50 * ( 0 - 25 ) / 166 => 255" severity error;
		a <= "011111111"; b <= "010000010"; c <= "011111110"; d <= "011111111"; e <= "000011010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 130 * ( 254 - 255 ) / 26 => 255" severity error;
		a <= "000000000"; b <= "000110010"; c <= "010101111"; d <= "010000000"; e <= "010000100";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000010001" report "0 + 50 * ( 175 - 128 ) / 132 => 17" severity error;
		a <= "011111111"; b <= "010000111"; c <= "000110010"; d <= "011111111"; e <= "111110011";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 135 * ( 50 - 255 ) / -13 => 255" severity error;
		a <= "011111111"; b <= "010001001"; c <= "011111110"; d <= "011111111"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 137 * ( 254 - 255 ) / 50 => 255" severity error;
		a <= "000011001"; b <= "010111101"; c <= "001111101"; d <= "000000000"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "25 + 189 * ( 125 - 0 ) / 50 => 255" severity error;
		a <= "011111111"; b <= "010001110"; c <= "001111101"; d <= "011111111"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 142 * ( 125 - 255 ) / 50 => 255" severity error;
		a <= "000011001"; b <= "010010110"; c <= "001001011"; d <= "010000000"; e <= "010010000";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "25 + 150 * ( 75 - 128 ) / 144 => 255" severity error;
		a <= "000000000"; b <= "000110010"; c <= "011111110"; d <= "000011001"; e <= "000001011";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "0 + 50 * ( 254 - 25 ) / 11 => 255" severity error;
		a <= "011111111"; b <= "010010101"; c <= "000110010"; d <= "011111111"; e <= "001101111";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 149 * ( 50 - 255 ) / 111 => 255" severity error;
		a <= "000000000"; b <= "000110010"; c <= "000110010"; d <= "000011001"; e <= "111100100";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "0 + 50 * ( 50 - 25 ) / -28 => 255" severity error;
		a <= "010000000"; b <= "010011010"; c <= "000000000"; d <= "000011001"; e <= "001001001";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "128 + 154 * ( 0 - 25 ) / 73 => 255" severity error;
		a <= "011111111"; b <= "010011100"; c <= "001111101"; d <= "011111111"; e <= "000110010";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "011111111" report "255 + 156 * ( 125 - 255 ) / 50 => 255" severity error;
		a <= "000011001"; b <= "000100010"; c <= "010101111"; d <= "010000000"; e <= "010011111";
		reset <= '1'; wait for 20 ns; reset <= '0'; wait for 600 ns;
		assert result = "000100011" report "25 + 34 * ( 175 - 128 ) / 159 => 35" severity error;

		report "Don't worry, it's just the end of the simulation." severity failure;
	end process;
	
end Test;
